{-# OPTIONS -#include "DLPrim_hsc.h" #-}
{-# LINE 1 "DLPrim.hsc" #-}
--  Haskell Binding for dl{open,sym,...}          -*-haskell-*-
{-# LINE 2 "DLPrim.hsc" #-}
--
--  Author : Volker Stolz <stolz@i2.informatik.rwth-aachen.de>
--
--  Created: 2001-11-22
--
--  Derived from GModule.chs by M.Weber & M.Chakravarty which is part of c2hs
--  I left the API more or less the same, mostly the flags are different.
--
--  License: BSD
--

module DLPrim (
  dlopen,
  dlsym,
  dlerror,
  dlclose,
  -- dlAddr, -- XXX NYI
  haveRtldNext,
  haveRtldLocal,
  packModuleFlags,
  ModuleFlags(..),
  Source(..)
 )

where

import Bits ((.|.))
import Ptr (Ptr, FunPtr, nullPtr, plusPtr)
import CTypes (CInt)
import CString (CString, withCString, peekCString)


{-# LINE 36 "DLPrim.hsc" #-}

{-# LINE 37 "DLPrim.hsc" #-}

{-# LINE 38 "DLPrim.hsc" #-}

-- RTLD_NEXT madness
-- On some host (e.g. SuSe Linux 7.2) RTLD_NEXT is not visible
-- without setting _GNU_SOURCE. Since we don't want to set this
-- flag, here's a different solution: You can use the Haskell
-- function 'haveRtldNext' to check wether the flag is available
-- to you. Ideally, this will be optimized by the compiler so
-- that it should be as efficient as an #ifdef.
--    If you fail to test the flag and use it although it is
-- undefined, 'packOneModuleFlag' will bomb.
--    The same applies to RTLD_LOCAL which isn't available on
-- cygwin.

haveRtldNext :: Bool


{-# LINE 60 "DLPrim.hsc" #-}
haveRtldNext = False

{-# LINE 62 "DLPrim.hsc" #-}

haveRtldLocal :: Bool


{-# LINE 66 "DLPrim.hsc" #-}
haveRtldLocal = True

{-# LINE 70 "DLPrim.hsc" #-}

-- data type definition
-- --------------------

-- flags passed to `moduleOpen' (EXPORTED)
--
data ModuleFlags 
  = RTLD_LAZY
  | RTLD_NOW
  | RTLD_GLOBAL 
  | RTLD_LOCAL
    deriving (Show, Read)

foreign import ccall unsafe "dlopen" dlopen :: CString -> CInt -> IO (Ptr ())
foreign import ccall unsafe "dlsym"  dlsym  :: (Ptr ()) -> CString -> IO (FunPtr a)
foreign import ccall unsafe "dlerror" dlerror :: IO CString
foreign import ccall unsafe "dlclose" dlclose :: (Ptr ()) -> IO ()

packModuleFlags :: [ModuleFlags] -> CInt
packModuleFlags flags = foldl (\ s f -> (packOneModuleFlag f) .|. s) 0 flags

packOneModuleFlag :: ModuleFlags -> CInt
packOneModuleFlag RTLD_LAZY = 1
{-# LINE 93 "DLPrim.hsc" #-}
packOneModuleFlag RTLD_NOW = 2
{-# LINE 94 "DLPrim.hsc" #-}
packOneModuleFlag RTLD_GLOBAL = 256
{-# LINE 95 "DLPrim.hsc" #-}

{-# LINE 96 "DLPrim.hsc" #-}
packOneModuleFlag RTLD_LOCAL = 0
{-# LINE 97 "DLPrim.hsc" #-}

{-# LINE 100 "DLPrim.hsc" #-}

data Source = Null | Next | Default | Name CString

packSource :: Source -> CString
packSource Null = nullPtr

{-# LINE 108 "DLPrim.hsc" #-}
packSource Next = error "RTLD_NEXT not available"

{-# LINE 110 "DLPrim.hsc" #-}
packSource Default = nullPtr
packSource (Name str) = str
